#! /usr/bin/python3
#! -*- coding:utf-8; mode:python; -*-

__all__ = [
    'NNFeature',
    'NNTestResult',
]

from .constants import *

class NNFeature(object):

    def __init__(self, mean : float, stdev : float, name : str = None):
        assert type(mean) is float
        assert type(stdev) is float
        assert name is None or type(name) is str
        self.name = name
        self.mean = mean
        self.stdev = stdev

class NNTestResult(object):

    def __init__(self, lhs : Id, rhs : Id, exp : float, act : float):
        assert type(lhs) is Id
        assert type(rhs) is Id
        assert type(exp) is float
        assert type(act) is float
        self.lhs = lhs
        self.rhs = rhs
        self.expected = exp
        self.actual = act

    @property
    def error(self):
        return self.actual - self.expected

    @property
    def status(self):
        if abs(self.expected) < 1.0E-2:
            return TestStatus.UNKNOWN
        elif self.expected > 0.0 and self.actual > 0.0:
            return TestStatus.TRUE_POSITIVE
        elif self.expected < 0.0 and self.actual < 0.0:
            return TestStatus.TRUE_NEGATIVE
        elif self.expected < 0.0 and self.actual > 0.0:
            return TestStatus.FALSE_POSITIVE
        elif self.expected > 0.0 and self.actual < 0.0:
            return TestStatus.FALSE_NEGATIVE
        else:
            raise AssertionError(repr((self.expected, self.actual)))

    def __bool__(self):
        return self.status is TestStatus.PASSED
