<?xml version="1.0" encoding="UTF-8"?>

<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" >

  <xsl:output method="html" indent="yes"/>

  <xsl:variable name="apos">&apos;</xsl:variable>
  <xsl:variable name="quot">&quot;</xsl:variable>

  <xsl:template match="root">
    <xsl:variable name="layoutid">
      <xsl:value-of select="id" />
    </xsl:variable>
    <xsl:variable name="graphid">
      <xsl:value-of select="graph/@id" />
    </xsl:variable>
    <xsl:variable name="layout">
      <xsl:choose>
        <xsl:when test="layouts/layout[@id = $layoutid]/@layout">
          <xsl:value-of select="layouts/layout[@id = $layoutid]/@layout" />
        </xsl:when>
        <xsl:otherwise>UNKNOWN</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <html>
      <head>
        <meta charset="UTF-8" />
        <title>Graphstudy</title>
        <link rel="shortcut icon" type="image/png" href="/graphstudy.png" />
        <link rel="stylesheet" type="text/css" href="/graphstudy.css" />
        <script type="text/javascript" src="/graphstudy.js"></script>
      </head>
      <body onload="init()">
        <xsl:call-template name="make-layouts">
          <xsl:with-param name="graphid" select="$graphid" />
          <xsl:with-param name="layoutid" select="$layoutid" />
        </xsl:call-template>
        <h1 class="constant-layouts"><xsl:value-of select="$layout" /></h1>
        <xsl:call-template name="make-summary">
          <xsl:with-param name="graphid" select="$graphid" />
          <xsl:with-param name="layoutid" select="$layoutid" />
        </xsl:call-template>
        <xsl:call-template name="make-picture">
          <xsl:with-param name="graphid" select="$graphid" />
          <xsl:with-param name="layoutid" select="$layoutid" />
        </xsl:call-template>
        <xsl:call-template name="make-worse" />
        <xsl:call-template name="make-princomp">
          <xsl:with-param name="graphid" select="$graphid" />
          <xsl:with-param name="layoutid" select="$layoutid" />
        </xsl:call-template>
        <xsl:call-template name="make-properties">
          <xsl:with-param name="graphid" select="$graphid" />
          <xsl:with-param name="layoutid" select="$layoutid" />
        </xsl:call-template>
      </body>
    </html>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-summary">
    <xsl:param name="graphid" />
    <xsl:param name="layoutid" />
    <h2>Summary</h2>
    <table class="horizontal">
      <tr>
        <th>Graph-ID</th>
        <td>
          <a>
            <xsl:attribute name="href"><xsl:value-of select="concat('/graphs/', $graphid, '/')" /></xsl:attribute>
            <xsl:value-of select="$graphid" />
          </a>
        </td>
      </tr>
      <tr>
        <th>Generator</th>
        <td><xsl:value-of select="graph/generator" /></td>
      </tr>
      <tr>
        <th>Size-Class</th>
        <td><xsl:value-of select="graph/size" /></td>
      </tr>
      <tr>
        <th>Nodes</th>
        <td class="number-int"><xsl:value-of select="graph/nodes" /></td>
      </tr>
      <tr>
        <th>Edges</th>
        <td class="number-int"><xsl:value-of select="graph/edges" /></td>
      </tr>
      <tr>
        <th>Layout-ID</th>
        <td>
          <a>
            <xsl:attribute name="href">
              <xsl:value-of select="concat('/layouts/', $layoutid, '/')" />
            </xsl:attribute>
            <xsl:value-of select="$layoutid" />
          </a>
        </td>
      </tr>
      <xsl:choose>
        <xsl:when test="layouts/layout[@id = $layoutid]/@layout">
          <tr>
            <th>Layout</th>
            <td><xsl:value-of select="layouts/layout[@id = $layoutid]/@layout" /></td>
          </tr>
        </xsl:when>
        <xsl:when test="interpolation">
          <tr>
            <th>Interpolation</th>
            <td><xsl:value-of select="interpolation/@method" /></td>
          </tr>
          <xsl:apply-templates select="interpolation/parent" />
          <tr>
            <th>Rate</th>
            <td><span class="number-percent"><xsl:value-of select="interpolation/@rate" /></span>&#x00A0;%</td>
          </tr>
        </xsl:when>
        <xsl:when test="worsening">
          <tr>
            <th>Worsening</th>
            <td><xsl:value-of select="worsening/@method" /></td>
          </tr>
          <xsl:apply-templates select="worsening/parent" />
          <tr>
            <th>Rate</th>
            <td><span class="number-percent"><xsl:value-of select="worsening/@rate" /></span>&#x00A0;%</td>
          </tr>
        </xsl:when>
        <xsl:otherwise>
          <xsl:message>I'm confused by this kind of layout.</xsl:message>
        </xsl:otherwise>
      </xsl:choose>
    </table>
  </xsl:template>

  <xsl:template match="parent">
    <xsl:param name="title" select="'Parent-ID'" />
    <tr>
      <th><xsl:value-of select="$title" /></th>
      <td>
        <a>
          <xsl:attribute name="href">
            <xsl:value-of select="concat('/layouts/', text(), '/')" />
          </xsl:attribute>
          <xsl:value-of select="." />
        </a>
      </td>
    </tr>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-picture">
    <xsl:param name="graphid" />
    <xsl:param name="layoutid" />
    <h2>Picture</h2>
    <p>
      <a>
        <xsl:attribute name="href">
          <xsl:value-of select="concat('/layouts/', $layoutid, '/picture.svg')" />
        </xsl:attribute>
        <img class="pretty">
          <xsl:attribute name="src">
            <xsl:value-of select="concat('/layouts/', $layoutid, '/picture.png')" />
          </xsl:attribute>
        </img>
      </a>
    </p>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-worse">
    <h2>Worse Layouts</h2>
    <xsl:for-each select="all-lay-worse/lay-worse">
      <xsl:sort select="@key" data-type="number" />
      <xsl:variable name="worse"><xsl:value-of select="." /></xsl:variable>
      <xsl:variable name="count">
        <xsl:value-of select="count(/root/worse-layouts/layout[@method = $worse])" />
      </xsl:variable>
      <h3><xsl:value-of select="$worse" /></h3>
      <p>
        <xsl:choose>
          <xsl:when test="$count = 0">
            <xsl:text>No thusly worsened layouts exist.</xsl:text>
          </xsl:when>
          <xsl:when test="$count = 1">
            <span class="number-int"><xsl:value-of select="$count" /></span>
            <xsl:text> thusly worsened layout exist.</xsl:text>
          </xsl:when>
          <xsl:otherwise>
            <span class="number-int"><xsl:value-of select="$count" /></span>
            <xsl:text> thusly worsened layouts exist.</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </p>
      <xsl:if test="$count &gt; 0">
        <ul class="layout-glider">
          <xsl:for-each select="/root/worse-layouts/layout[@method = $worse]">
            <xsl:sort select="@rate" data-type="number" />
            <li>
              <xsl:attribute name="data-rate">
                <xsl:value-of select="@rate" />
              </xsl:attribute>
              <xsl:attribute name="data-preview">
                <xsl:value-of select="concat('/layouts/', @id, '/thumbnail.png')" />
              </xsl:attribute>
              <a>
                <xsl:attribute name="href">
                  <xsl:value-of select="concat('/layouts/', @id, '/')" />
                </xsl:attribute>
                <xsl:value-of select="@id" />
              </a>
            </li>
          </xsl:for-each>
        </ul>
      </xsl:if>
    </xsl:for-each>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-princomp">
    <xsl:param name="graphid" />
    <xsl:param name="layoutid" />
    <h2>Principial Axes</h2>
    <xsl:choose>
      <xsl:when test="princomp">
        <table class="horizontal">
          <tr>
            <th>Major</th>
            <td>(</td>
            <td class="number-float" style="text-align:right"><xsl:value-of select="princomp/major/value[1]" /></td>
            <td>,</td>
            <td class="number-float" style="text-align:right"><xsl:value-of select="princomp/major/value[2]" /></td>
            <td>)</td>
          </tr>
          <tr>
            <th>Minor</th>
            <td>(</td>
            <td class="number-float" style="text-align:right"><xsl:value-of select="princomp/minor/value[1]" /></td>
            <td>,</td>
            <td class="number-float" style="text-align:right"><xsl:value-of select="princomp/minor/value[2]" /></td>
            <td>)</td>
          </tr>
        </table>
        <p>
          <xsl:text>View a </xsl:text>
          <a>
            <xsl:attribute name="href">
              <xsl:value-of select="concat('/layouts/', $layoutid, '/princomp.svg')" />
            </xsl:attribute>
            <xsl:text>drawing</xsl:text>
          </a>
          <xsl:text> of the layout's principial axes.</xsl:text>
        </p>
      </xsl:when>
      <xsl:otherwise>
        <p class="alert">Sorry, no principial component analysis available.</p>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-properties">
    <xsl:param name="graphid" />
    <xsl:param name="layoutid" />
    <xsl:for-each select="all-properties/property">
      <xsl:sort select="@key" data-type="number" />
      <xsl:variable name="prop"><xsl:value-of select="." /></xsl:variable>
      <xsl:variable name="unit"><xsl:value-of select="'AU'" /></xsl:variable>  <!-- TODO: actual unit -->
      <xsl:variable name="prop-json">
        <xsl:value-of select="translate($prop, '_ABCDEFGHIJKLMNOPQRSTUVWXYZ', '-abcdefghijklmnopqrstuvwxyz')" />
      </xsl:variable>
      <h2>
        <span class="constant-properties"><xsl:value-of select="$prop" /></span>
        <xsl:text> </xsl:text>
        <xsl:text>(</xsl:text>
        <a>
          <xsl:attribute name="href">
            <xsl:value-of select="concat('/properties/', $graphid, '/', $prop-json, '/')" />
          </xsl:attribute>
          <xsl:value-of select="$prop" />
        </a>
        <xsl:text>)</xsl:text>
      </h2>
      <xsl:if test="not(/root/properties/property-data[@property = $prop])">
        <p class="alert">Sorry, no data for property <xsl:value-of select="$prop" /> is avilable for this layout.</p>
      </xsl:if>
      <xsl:apply-templates select="/root/properties/property-data[@property = $prop]">
        <xsl:with-param name="unit" select="$unit" />
        <xsl:with-param name="layoutid" select="$layoutid" />
        <xsl:with-param name="prop" select="$prop" />
      </xsl:apply-templates>
    </xsl:for-each>
  </xsl:template>

  <xsl:template match="property-data">
    <xsl:param name="prop" />
    <p>Consider yourself lucky: data for property <xsl:value-of select="$prop" /> is avilable for this layout!</p>
  </xsl:template>

  <xsl:template match="property-data[@kind = 'SCALED_HISTOGRAM']">
    <xsl:param name="unit" />
    <xsl:param name="layoutid" />
    <xsl:param name="prop" />
    <xsl:call-template name="scaled-histogram-common">
      <xsl:with-param name="layoutid" select="$layoutid" />
      <xsl:with-param name="unit" select="$unit" />
    </xsl:call-template>
    <p>
      <xsl:text>There are </xsl:text>
      <span class="number-int"><xsl:value-of select="count(histogram)" /></span>
      <xsl:text> histograms for this layout and property.</xsl:text>
    </p>
    <xsl:call-template name="scaled-histogram-table">
      <xsl:with-param name="unit" select="$unit" />
    </xsl:call-template>
    <xsl:call-template name="scaled-histogram-figure">
      <xsl:with-param name="layoutid" select="$layoutid" />
      <xsl:with-param name="prop" select="$prop" />
    </xsl:call-template>
  </xsl:template>

  <xsl:template match="property-data[@kind = 'GLIDING_AVERAGE']">
    <xsl:param name="unit" />
    <xsl:param name="layoutid" />
    <xsl:param name="prop" />
    <xsl:call-template name="gliding-average-table">
      <xsl:with-param name="unit" select="$unit" />
    </xsl:call-template>
    <xsl:call-template name="gliding-average-figure">
      <xsl:with-param name="layoutid" select="$layoutid" />
      <xsl:with-param name="prop" select="$prop" />
    </xsl:call-template>
  </xsl:template>

  <!-- context: property-data[@kind = 'SCALED_HISTOGRAM'] -->
  <xsl:template name="scaled-histogram-common">
    <xsl:param name="layoutid" />
    <xsl:param name="unit" />
    <table class="horizontal">
      <tr>
        <th>Size</th>
        <td class="number-int" style="text-align:right"><xsl:value-of select="population/size" /></td>
        <td></td>
      </tr>
      <tr>
        <th>Minimum</th>
        <td class="number-float" style="text-align:right"><xsl:value-of select="population/minimum" /></td>
        <td><xsl:value-of select="$unit" /></td>
      </tr>
      <tr>
        <th>Maximum</th>
        <td class="number-float" style="text-align:right"><xsl:value-of select="population/maximum" /></td>
        <td><xsl:value-of select="$unit" /></td>
      </tr>
      <tr>
        <th>Average</th>
        <td class="number-float" style="text-align:right"><xsl:value-of select="population/mean" /></td>
        <td><xsl:value-of select="$unit" /></td>
      </tr>
      <tr>
        <th>RMS</th>
        <td class="number-float" style="text-align:right"><xsl:value-of select="population/rms" /></td>
        <td><xsl:value-of select="$unit" /></td>
      </tr>
      <tr>
        <th>Entropy</th>
        <td class="number-float" style="text-align:right"><xsl:value-of select="population/entropy" /></td>
        <td>bit</td>
      </tr>
    </table>
  </xsl:template>

  <!-- context: property-data[@kind = 'SCALED_HISTOGRAM'] -->
  <xsl:template name="scaled-histogram-table">
    <xsl:param name="unit" />
    <table class="pretty">
      <tr>
        <th data-sort-type="d">Bin Count</th>
        <th data-sort-type="f">Bin Width</th>
        <th data-sort-type="f">Entropy</th>
        <th data-sort-type="d">Binning</th>
      </tr>
      <tr>
        <th>1</th>
        <th><xsl:value-of select="$unit" /></th>
        <th>bit</th>
        <th>&#x2014;</th>
      </tr>
      <xsl:for-each select="histogram">
        <xsl:sort select="bincount" data-type="number" />
        <tr>
          <td class="number-int"><xsl:value-of select="bincount" /></td>
          <td class="number-float"><xsl:value-of select="binwidth" /></td>
          <td class="number-float"><xsl:value-of select="entropy" /></td>
          <td>
            <xsl:attribute name="data-sort-key">
              <xsl:value-of select="binning/@key" />
            </xsl:attribute>
            <xsl:value-of select="binning" />
          </td>
        </tr>
      </xsl:for-each>
    </table>
  </xsl:template>

  <!-- context: property-data[@kind = 'GLIDING_AVERAGE'] -->
  <xsl:template name="gliding-average-table">
    <xsl:param name="unit" />
    <table class="pretty">
      <tr>
        <th data-sort-type="d">Vicinity</th>
        <th data-sort-type="d">Size</th>
        <th data-sort-type="f">Minimum</th>
        <th data-sort-type="f">Maximum</th>
        <th data-sort-type="f">Average</th>
        <th data-sort-type="f">RMS</th>
        <th data-sort-type="f">Sigma</th>
        <th data-sort-type="d">Points</th>
        <th data-sort-type="f">Entropy</th>
      </tr>
      <tr>
        <th>1</th>
        <th>1</th>
        <th><xsl:value-of select="$unit" /></th>
        <th><xsl:value-of select="$unit" /></th>
        <th><xsl:value-of select="$unit" /></th>
        <th><xsl:value-of select="$unit" /></th>
        <th><xsl:value-of select="$unit" /></th>
        <th>1</th>
        <th>bit</th>
      </tr>
      <xsl:for-each select="density">
        <xsl:sort select="vicinity" data-type="number" />
        <tr>
          <td class="number-int"><xsl:value-of select="vicinity" /></td>
          <td class="number-int"><xsl:value-of select="size" /></td>
          <td class="number-float"><xsl:value-of select="minimum" /></td>
          <td class="number-float"><xsl:value-of select="maximum" /></td>
          <td class="number-float"><xsl:value-of select="mean" /></td>
          <td class="number-float"><xsl:value-of select="rms" /></td>
          <td class="number-float"><xsl:value-of select="sigma" /></td>
          <td class="number-int"><xsl:value-of select="points" /></td>
          <td class="number-float"><xsl:value-of select="entropy" /></td>
        </tr>
      </xsl:for-each>
    </table>
  </xsl:template>

  <!-- context: property-data[@kind = 'SCALED_HISTOGRAM'] -->
  <xsl:template name="scaled-histogram-figure">
    <xsl:param name="layoutid" />
    <xsl:param name="prop" />
    <xsl:variable name="prop-json">
      <xsl:value-of select="translate($prop, '_ABCDEFGHIJKLMNOPQRSTUVWXYZ', '-abcdefghijklmnopqrstuvwxyz')" />
    </xsl:variable>
    <xsl:variable name="ctrlid">
      <xsl:value-of select="concat($prop-json, '-histo-control-anchor')" />
    </xsl:variable>
    <xsl:variable name="histoid">
      <xsl:value-of select="concat($prop-json, '-histo')" />
    </xsl:variable>
    <xsl:variable name="defaultcount">
      <xsl:choose>
        <xsl:when test="histogram[binning = 'SCOTT_NORMAL_REFERENCE']/bincount">
          <xsl:value-of select="histogram[binning = 'SCOTT_NORMAL_REFERENCE']/bincount" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="histogram[1]/bincount" />
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <xsl:variable name="haveauto">
      <xsl:choose>
        <xsl:when test="histogram[binning = 'SCOTT_NORMAL_REFERENCE']/bincount">1</xsl:when>
        <xsl:otherwise>0</xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <p>
      <xsl:attribute name="id"><xsl:value-of select="concat($prop-json, '-histo-control-anchor')" /></xsl:attribute>
      <xsl:text>Number of histogram bins:</xsl:text>
      <xsl:text>&#x20;</xsl:text>
      <select>
        <xsl:attribute name="onchange">
          <xsl:value-of select="concat('updateHistogram(', $apos, $histoid, $apos, ', this.value)')" />
        </xsl:attribute>
        <xsl:if test="$haveauto &gt; 0">
          <option disabled="disabled" selected="selected">auto</option>
        </xsl:if>
        <xsl:for-each select="histogram">
          <xsl:sort select="bincount" data-type="number" />
          <xsl:if test="binning = 'FIXED_COUNT'">
            <option>
              <xsl:attribute name="value">
                <xsl:value-of select="concat('/property-plots/', $prop-json, '/histo/', $layoutid, '.svg?bincount=', bincount)" />
              </xsl:attribute>
              <xsl:value-of select="bincount" />
            </option>
          </xsl:if>
        </xsl:for-each>
      </select>
      <a class="anchor"><xsl:attribute name="href">
        <xsl:value-of select="concat('#', $ctrlid)" /></xsl:attribute>
        <xsl:text>&#x00b6;</xsl:text>
      </a>
    </p>
    <p>
      <img class="pretty">
        <xsl:attribute name="id"><xsl:value-of select="$histoid" /></xsl:attribute>
        <xsl:attribute name="data-property-name"><xsl:value-of select="$prop" /></xsl:attribute>
        <xsl:attribute name="src">
          <xsl:value-of select="concat('/property-plots/', $prop-json, '/histo/', $layoutid, '.svg?bincount=', $defaultcount)" />
        </xsl:attribute>
      </img>
    </p>
  </xsl:template>

  <!-- context: property-data[@kind = 'GLIDING_AVERAGE'] -->
  <xsl:template name="gliding-average-figure">
    <xsl:param name="layoutid" />
    <xsl:param name="prop" />
    <xsl:variable name="prop-json">
      <xsl:value-of select="translate($prop, '_ABCDEFGHIJKLMNOPQRSTUVWXYZ', '-abcdefghijklmnopqrstuvwxyz')" />
    </xsl:variable>
    <xsl:variable name="ctrlid">
      <xsl:value-of select="concat($prop-json, '-gliding-control-anchor')" />
    </xsl:variable>
    <xsl:variable name="glidingid">
      <xsl:value-of select="concat($prop-json, '-gliding')" />
    </xsl:variable>
    <p>
      <xsl:attribute name="id"><xsl:value-of select="concat($prop-json, '-gliding-control-anchor')" /></xsl:attribute>
      <xsl:text>Vicinity:</xsl:text>
      <xsl:text>&#x20;</xsl:text>
      <select>
        <xsl:attribute name="onchange">
          <xsl:value-of select="concat('updateGliding(', $apos, $glidingid, $apos, ', this.value)')" />
        </xsl:attribute>
        <option selected="selected">
          <xsl:attribute name="value">
            <xsl:value-of select="concat('/property-plots/', $prop-json, '/gliding/', $layoutid, '.svg')" />
          </xsl:attribute>
          <xsl:text>all</xsl:text>
        </option>
        <xsl:for-each select="density">
          <xsl:sort select="vicinity" data-type="number" />
          <option>
            <xsl:attribute name="value">
              <xsl:value-of select="concat('/property-plots/', $prop-json, '/gliding/', $layoutid, '.svg?vicinity=', vicinity)" />
            </xsl:attribute>
            <xsl:value-of select="vicinity" />
          </option>
        </xsl:for-each>
      </select>
      <a class="anchor"><xsl:attribute name="href">
        <xsl:value-of select="concat('#', $ctrlid)" /></xsl:attribute>
        <xsl:text>&#x00b6;</xsl:text>
      </a>
    </p>
    <p>
      <img class="pretty">
        <xsl:attribute name="id"><xsl:value-of select="$glidingid" /></xsl:attribute>
        <xsl:attribute name="data-property-name"><xsl:value-of select="$prop" /></xsl:attribute>
        <xsl:attribute name="src">
          <xsl:value-of select="concat('/property-plots/', $prop-json, '/gliding/', $layoutid, '.svg')" />
        </xsl:attribute>
      </img>
    </p>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-layouts">
    <xsl:param name="graphid" />
    <xsl:param name="layoutid" />
    <p class="nocss">All layouts for this graph:</p>
    <ul class="top-navigation">
      <xsl:for-each select="all-layouts/layout">
        <xsl:sort select="@key" data-type="number" />
        <xsl:variable name="layout">
          <xsl:value-of select="." />
        </xsl:variable>
        <li>
          <xsl:if test="/root/layouts/layout[@layout = $layout]/@id = $layoutid">
            <xsl:attribute name="class">current</xsl:attribute>
          </xsl:if>
          <xsl:choose>
            <xsl:when test="/root/layouts/layout[@layout = $layout]">
              <a>
                <xsl:attribute name="href">
                  <xsl:value-of select="concat('/layouts/', /root/layouts/layout[@layout = $layout]/@id)" />
                </xsl:attribute>
                <xsl:attribute name="title">
                  <xsl:value-of select="/root/layouts/layout[@layout = $layout]/@id" />
                </xsl:attribute>
                <xsl:value-of select="." />
              </a>
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="." />
            </xsl:otherwise>
          </xsl:choose>
        </li>
      </xsl:for-each>
    </ul>
  </xsl:template>

</xsl:transform>
