<?xml version="1.0" encoding="UTF-8"?>

<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" >

  <xsl:output method="html" indent="yes"/>

  <xsl:template match="root">
    <html>
      <head>
        <meta charset="UTF-8" />
        <title>Graphstudy</title>
        <link rel="shortcut icon" type="image/png" href="/graphstudy.png" />
        <link rel="stylesheet" type="text/css" href="/graphstudy.css" />
        <script type="text/javascript" src="/graphstudy.js" />
        <script type="text/javascript" src="/nn/testscore.js" />
      </head>
      <body onload="init(); NN.init();">
        <h1>Neural Network Test Score</h1>
        <p>View this information <a href="?graphics=false">tabulated</a>.</p>
        <p id="something-went-wrong" class="alert">
          Sorry, the test case you're looking for does not seem to exist or your browser might experience trouble
          displaying this page.  If you reckon that the latter might be the case, please make sure that you have CSS and
          JavaScript support enabled.
        </p>
        <xsl:apply-templates select="test-cases/test-case">
          <xsl:with-param name="total" select="count(test-cases/test-case)" />
        </xsl:apply-templates>
        <p id="scroll-for-more" style="display:none">Scroll down to show more test cases</p>
        <div id="vertical-fill" />
      </body>
    </html>
  </xsl:template>

  <xsl:template match="test-case">
    <xsl:param name="total" />
    <div style="display:none">
      <h2>
        <xsl:attribute name="id">
          <xsl:value-of select="concat('test-case-', position())" />
        </xsl:attribute>
        <xsl:text>[</xsl:text>
        <span class="number-int"><xsl:value-of select="position()" /></span>
        <xsl:text> / </xsl:text>
        <span class="number-int"><xsl:value-of select="$total" /></span>
        <xsl:text>] </xsl:text>
        <span class="constant-test-status"><xsl:value-of select="@status" /></span>
        <a class="anchor" title="link to this test case">
          <xsl:attribute name="href">
            <xsl:value-of select="concat('#test-case-', position())" />
          </xsl:attribute>
          <xsl:text>&#x00A7;</xsl:text>
        </a>
      </h2>
      <p>
        <xsl:text>Compare the </xsl:text>
        <a>
          <xsl:attribute name="href">
            <xsl:value-of select="concat('/nn/features/?lhs=', lhs, ';rhs=', rhs)" />
          </xsl:attribute>
          <xsl:text>feature vectors</xsl:text>
        </a>
        <xsl:text> of the two layouts.</xsl:text>
      </p>
      <div class="wannabe-layout-comparison">
        <xsl:attribute name="data-test-case">
          <xsl:value-of select="position()" />
        </xsl:attribute>
        <xsl:attribute name="data-info">
          <xsl:call-template name="test-case-to-json" />
        </xsl:attribute>
      </div>
    </div>
  </xsl:template>

  <xsl:template match="lhs|rhs">
    <a>
      <xsl:attribute name="href">
        <xsl:value-of select="concat('/layouts/', text(), '/')" />
      </xsl:attribute>
      <img>
        <xsl:attribute name="src">
          <xsl:value-of select="concat('/layouts/', text(), '/picture.png')" />
        </xsl:attribute>
      </img>
    </a>
  </xsl:template>

  <xsl:template name="test-case-to-json">
    <xsl:variable name="quot">&quot;</xsl:variable>
    <xsl:text>{ </xsl:text>
    <xsl:text>"lhs" : </xsl:text>
    <xsl:value-of select="concat($quot, lhs, $quot)" />
    <xsl:text>, </xsl:text>
    <xsl:text>"rhs" : </xsl:text>
    <xsl:value-of select="concat($quot, rhs, $quot)" />
    <xsl:text>, </xsl:text>
    <xsl:text>"lhsInformal" : </xsl:text>
    <xsl:value-of select="concat($quot, lhs/@informal, $quot)" />
    <xsl:text>, </xsl:text>
    <xsl:text>"rhsInformal" : </xsl:text>
    <xsl:value-of select="concat($quot, rhs/@informal, $quot)" />
    <xsl:text>, </xsl:text>
    <xsl:text>"expected" : </xsl:text>
    <xsl:value-of select="expected" />
    <xsl:text>, </xsl:text>
    <xsl:text>"actual" : </xsl:text>
    <xsl:value-of select="actual" />
    <xsl:text>, </xsl:text>
    <xsl:text>"status" : </xsl:text>
    <xsl:value-of select="concat($quot, @status, $quot)" />
    <xsl:text> }</xsl:text>
  </xsl:template>

</xsl:transform>
