<?xml version="1.0" encoding="UTF-8"?>

<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" >

  <xsl:output method="html" indent="yes"/>

  <xsl:variable name="apos">&apos;</xsl:variable>
  <xsl:variable name="quot">&quot;</xsl:variable>

  <xsl:template match="root">
    <xsl:variable name="graphid">
      <xsl:value-of select="graph/@id" />
    </xsl:variable>
    <xsl:variable name="prop">
      <xsl:value-of select="property" />
    </xsl:variable>
    <xsl:variable name="prop-json">
      <xsl:value-of select="translate(property, '_ABCDEFGHIJKLMNOPQRSTUVWXYZ', '-abcdefghijklmnopqrstuvwxyz')" />
    </xsl:variable>
    <html>
      <head>
        <meta charset="UTF-8" />
        <title>Graphstudy</title>
        <link rel="shortcut icon" type="image/png" href="/graphstudy.png" />
        <link rel="stylesheet" type="text/css" href="/graphstudy.css" />
        <script type="text/javascript" src="/graphstudy.js"></script>
      </head>
      <body onload="init()">
        <xsl:apply-templates select="all-properties">
          <xsl:with-param name="graphid" select="$graphid" />
          <xsl:with-param name="property" select="$prop" />
        </xsl:apply-templates>
        <h1 class="constant-properties"><xsl:value-of select="$prop" /></h1>
        <xsl:apply-templates select="graph" />
        <xsl:apply-templates select="all-layouts" />
        <xsl:choose>
          <xsl:when test="property/@kind = 'SCALED_HISTOGRAM'">
            <h2>Entropy (Scaled Histogram)</h2>
            <xsl:call-template name="make-entropy-table-scaled-histogram">
              <xsl:with-param name="graphid" select="$graphid" />
              <xsl:with-param name="prop-json" select="$prop-json" />
            </xsl:call-template>
            <xsl:call-template name="make-entropy-figure">
              <xsl:with-param name="graphid" select="$graphid" />
              <xsl:with-param name="prop-json" select="$prop-json" />
            </xsl:call-template>
          </xsl:when>
          <xsl:when test="property/@kind = 'GLIDING_AVERAGE'">
            <h2>Entropy (Gliding Average)</h2>
            <xsl:call-template name="make-entropy-table-gliding-average">
              <xsl:with-param name="graphid" select="$graphid" />
              <xsl:with-param name="prop-json" select="$prop-json" />
            </xsl:call-template>
            <xsl:call-template name="make-entropy-figure">
              <xsl:with-param name="graphid" select="$graphid" />
              <xsl:with-param name="prop-json" select="$prop-json" />
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <p class="alert">Sorry, the property kind <xsl:value-of select="property/@kind" /> is unknown to me.</p>
          </xsl:otherwise>
        </xsl:choose>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="graph">
    <h2>Summary</h2>
    <table class="horizontal">
      <tr>
        <th>Graph-ID</th>
        <td>
          <a>
            <xsl:attribute name="href"><xsl:value-of select="concat('/graphs/', @id, '/')" /></xsl:attribute>
            <xsl:value-of select="@id" />
          </a>
        </td>
      </tr>
      <tr>
        <th>Generator</th>
        <td><xsl:value-of select="generator" /></td>
      </tr>
      <tr>
        <th>Size-Class</th>
        <td><xsl:value-of select="size" /></td>
      </tr>
      <tr>
        <th>Nodes</th>
        <td class="number-int"><xsl:value-of select="nodes" /></td>
      </tr>
      <tr>
        <th>Edges</th>
        <td class="number-int"><xsl:value-of select="edges" /></td>
      </tr>
    </table>
  </xsl:template>

  <xsl:template match="all-layouts">
    <h2>Layouts</h2>
    <form action="" method="GET">
      <fieldset>
        <legend>Select all layouts of this graph to be included in the comparison.</legend>
        <xsl:for-each select="layout">
          <xsl:sort select="@key" data-type="number" />
          <xsl:variable name="layout"><xsl:value-of select="." /></xsl:variable>
          <xsl:variable name="checkid">
            <xsl:value-of select="concat('check-', translate($layout, '_ABCDEFGHIJKLMNOPQRSTUVWXYZ', '-abcdefghijklmnopqrstuvwxyz'))" />
          </xsl:variable>
          <div>
            <xsl:attribute name="id">
              <xsl:value-of select="$checkid" />
            </xsl:attribute>
            <input type="checkbox" name="layouts">
              <xsl:choose>
                <xsl:when test="../../layouts/layout-data[@layout = $layout]">
                  <xsl:attribute name="value">
                    <xsl:value-of select="../../layouts/layout-data[@layout = $layout]/@id" />
                  </xsl:attribute>
                  <xsl:if test="boolean(number(../../layouts/layout-data[@layout = $layout]/@selected))">
                    <xsl:attribute name="checked" />
                  </xsl:if>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:attribute name="disabled" />
                  <xsl:attribute name="title">No property computed or layout does not exist.</xsl:attribute>
                </xsl:otherwise>
              </xsl:choose>
            </input>
            <label>
            <xsl:attribute name="for">
              <xsl:value-of select="$checkid" />
            </xsl:attribute>
            <xsl:value-of select="." />
            </label>
          </div>
        </xsl:for-each>
      </fieldset>
      <p>
        <input type="button" class="jsonly" disabled="disabled" value="All" onclick="checkAllCheckBoxes('layouts', true)" />
        <xsl:text> </xsl:text>
        <input type="button" class="jsonly" disabled="disabled" value="None" onclick="checkAllCheckBoxes('layouts', false)" />
        <xsl:text> </xsl:text>
        <input type="submit" value="Update" />
      </p>
    </form>
  </xsl:template>

  <xsl:template match="all-properties">
    <xsl:param name="graphid" />
    <xsl:param name="property" />
    <p class="nocss">All properties for this graph:</p>
    <ul class="top-navigation">
      <xsl:for-each select="property">
        <xsl:sort select="@key" data-type="number" />
        <xsl:variable name="prop-json">
          <xsl:value-of select="translate(text(), '_ABCDEFGHIJKLMNOPQRSTUVWXYZ', '-abcdefghijklmnopqrstuvwxyz')" />
        </xsl:variable>
        <li>
          <xsl:if test="text() = $property">
            <xsl:attribute name="class">current</xsl:attribute>
          </xsl:if>
          <a>
            <xsl:attribute name="href">
              <xsl:value-of select="concat('/properties/', $graphid, '/', $prop-json, '/')" />
            </xsl:attribute>
            <xsl:value-of select="." />
          </a>
        </li>
      </xsl:for-each>
    </ul>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-entropy-table-scaled-histogram">
    <xsl:param name="graphid" />
    <xsl:param name="prop-json" />
    <table class="pretty">
      <tr>
        <th data-sort-type="s">Layout-ID</th>
        <th data-sort-type="d">Layout</th>
        <th colspan="2">Scott's Normal Reference</th>
        <xsl:if test="/root/all-bincounts/bincount">
          <th>
            <xsl:attribute name="colspan"><xsl:value-of select="count(/root/all-bincounts/bincount)" /></xsl:attribute>
            <xsl:text>Entropy (by bin count)</xsl:text>
          </th>
        </xsl:if>
      </tr>
      <tr>
        <th>&#x2014;</th>
        <th>&#x2014;</th>
        <th data-sort-type="d">Bin Count</th>
        <th data-sort-type="f">Entropy</th>
        <xsl:for-each select="/root/all-bincounts/bincount">
          <th data-sort-type="d"><xsl:value-of select="." /></th>
        </xsl:for-each>
      </tr>
      <xsl:for-each select="layouts/layout-data[boolean(number(@selected))]">
        <xsl:sort select="@id" />
        <xsl:variable name="layoutid">
          <xsl:value-of select="@id" />
        </xsl:variable>
        <xsl:variable name="layout">
            <xsl:value-of select="@layout" />
        </xsl:variable>
        <tr>
          <xsl:attribute name="id"><xsl:value-of select="@id" /></xsl:attribute>
          <td>
            <a>
              <xsl:attribute name="href">
                <xsl:value-of select="concat('/layouts/', @id, '/')" />
              </xsl:attribute>
              <xsl:value-of select="substring(@id, 1, 8)" />
            </a>
          </td>
          <td>
            <xsl:attribute name="data-sort-key">
              <xsl:value-of select="/root/all-layouts/layout[text() = $layout]/@key" />
            </xsl:attribute>
            <xsl:value-of select="$layout" />
          </td>
          <td class="number-int">
            <xsl:value-of select="entropy[@binning = 'SCOTT_NORMAL_REFERENCE']/@bincount" />
          </td>
          <td class="number-float">
            <xsl:value-of select="entropy[@binning = 'SCOTT_NORMAL_REFERENCE']" />
          </td>
          <xsl:for-each select="/root/all-bincounts/bincount">
            <xsl:sort select="." data-type="number" />
            <xsl:variable name="bincount">
              <xsl:value-of select="." />
            </xsl:variable>
            <td class="number-float">
              <xsl:value-of select="/root/layouts/layout-data[@id = $layoutid]/entropy[@binning = 'FIXED_COUNT' and @bincount = $bincount]" />
            </td>
          </xsl:for-each>
        </tr>
      </xsl:for-each>
    </table>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-entropy-table-gliding-average">
    <xsl:param name="graphid" />
    <xsl:param name="prop-json" />
    <table class="pretty">
      <tr>
        <th data-sort-type="s">Layout-ID</th>
        <th data-sort-type="d">Layout</th>
        <xsl:if test="/root/all-vicinities/vicinity">
          <th>
            <xsl:attribute name="colspan"><xsl:value-of select="count(/root/all-vicinities/vicinity)" /></xsl:attribute>
            <xsl:text>Entropy (by vicinity)</xsl:text>
          </th>
        </xsl:if>
      </tr>
      <tr>
        <th>&#x2014;</th>
        <th>&#x2014;</th>
        <xsl:for-each select="/root/all-vicinities/vicinity">
          <th data-sort-type="d"><xsl:value-of select="." /></th>
        </xsl:for-each>
      </tr>
      <xsl:for-each select="layouts/layout-data[boolean(number(@selected))]">
        <xsl:sort select="@id" />
        <xsl:variable name="layoutid">
          <xsl:value-of select="@id" />
        </xsl:variable>
        <xsl:variable name="layout">
            <xsl:value-of select="@layout" />
        </xsl:variable>
        <tr>
          <xsl:attribute name="id"><xsl:value-of select="@id" /></xsl:attribute>
          <td>
            <a>
              <xsl:attribute name="href">
                <xsl:value-of select="concat('/layouts/', @id, '/')" />
              </xsl:attribute>
              <xsl:value-of select="substring(@id, 1, 8)" />
            </a>
          </td>
          <td>
            <xsl:attribute name="data-sort-key">
              <xsl:value-of select="/root/all-layouts/layout[text() = $layout]/@key" />
            </xsl:attribute>
            <xsl:value-of select="$layout" />
          </td>
          <xsl:for-each select="/root/all-vicinities/vicinity">
            <xsl:sort select="." data-type="number" />
            <xsl:variable name="vicinity">
              <xsl:value-of select="." />
            </xsl:variable>
            <td class="number-float">
              <xsl:value-of select="/root/layouts/layout-data[@id = $layoutid]/entropy[@vicinity = $vicinity]" />
            </td>
          </xsl:for-each>
        </tr>
      </xsl:for-each>
    </table>
  </xsl:template>

  <!-- context: /root -->
  <xsl:template name="make-entropy-figure">
    <xsl:param name="graphid" />
    <xsl:param name="prop-json" />
    <p>
      <img class="pretty">
        <xsl:attribute name="src">
          <xsl:variable name="ploturl">
            <xsl:value-of select="concat('/property-plots/', $prop-json, '/entropy/', $graphid, '.svg')" />
          </xsl:variable>
          <xsl:variable name="plotquery">
            <xsl:apply-templates select="/root/layouts/layout-data[boolean(number(@selected))]" />
          </xsl:variable>
          <xsl:variable name="plotqueryall">
            <xsl:apply-templates select="/root/layout" />
          </xsl:variable>
          <xsl:choose>
            <xsl:when test="$plotquery = $plotqueryall">
              <xsl:value-of select="$ploturl" />
            </xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="concat($ploturl, '?', $plotquery)" />
            </xsl:otherwise>
          </xsl:choose>
        </xsl:attribute>
      </img>
    </p>
  </xsl:template>

  <xsl:template name="make-plot-query-string" match="/root/layouts/layout-data">
    <xsl:value-of select="concat('layouts=', @id)" />
    <xsl:if test="not(position() = last())">
      <xsl:text>&#x3B;</xsl:text>
    </xsl:if>
  </xsl:template>

</xsl:transform>
