#! /usr/bin/python3
#! -*- coding:utf-8; mode:python; -*-

__all__ = [
    'XJsonError',
    'load_xjson_file',
    'load_xjson_string',
]

import json
import logging

from .errors import *

class XJsonError(Error):

    def __init__(self, filename=None):
        if filename is not None:
            super().__init__("Cannot read / decode JSON file {!r}".format(filename))
        else:
            super().__init__("Cannot read / decode JSON file")

def load_xjson_file(source):
    if isinstance(source, str):
        try:
            with open(source, 'r') as istr:
                return load_xjson_file(istr)
        except (OSError, UnicodeDecodeError) as e:
            logging.error("{:s}: {!s}".format(source, e))
            raise XJsonError()
    filename = getattr(source, 'name', '/dev/stdin')
    try:
        return _load_xjson_pp(map(_preprocess_line, source), filename=filename)
    except (OSError, UnicodeDecodeError) as e:
            logging.error("{:s}: {!s}".format(filename, e))
            raise XJsonError()
    return _load_xjson_ppline(pplines, filename=filename)

def load_xjson_string(text, filename=None):
    return _load_xjson_pp(map(_preprocess_line, text.splitlines()), filename=filename)

def _load_xjson_pp(ppiter, filename=None):
    jsontext = '\n'.join(ppiter)
    if all(map(str.isspace, jsontext)):
        return None
    try:
        return json.loads(jsontext)
    except json.JSONDecodeError as e:
        filename = filename if filename is not None else '/dev/stdin'
        logging.error("{:s}:{:d}:{:d}: JSON syntax error: {!s}".format(filename, e.lineno, e.colno, e.msg))
        raise XJsonError(filename=filename)

def _preprocess_line(line):
    (prefix, slashes, suffix) = line.partition('//')
    return '' if slashes == '//' and not prefix.strip() else line.rstrip()
