#! /usr/bin/python3
#! -*- coding:utf-8; mode:python; -*-

import argparse

def main():
    ap = argparse.ArgumentParser(description="Creates a `properties.cfg` file.")
    ap.add_argument('properties', metavar='PROP', nargs='*', help="list of all properties")
    ap.add_argument('-o', '--output', metavar='FILE', help="write output to FILE")
    ap.add_argument('-x', '--exclude', metavar='PROP', action='append', default=list(), help="exclude PROP")
    ns = ap.parse_args()
    exclude = frozenset(canonical(s) for s in ns.exclude)
    properties = sorted({canonical(s) for s in ns.properties} - exclude)
    if ns.output is None or ns.output == '-':
        print_properties(properties)
    else:
        with open(ns.output, 'w') as ostr:
            print_properties(properties, ostr)

def print_properties(properties, ostr=None):
    for prop in properties:
        print(prop, file=ostr)

def canonical(s):
    return s.strip().replace('-', '_').upper()

if __name__ == '__main__':
    main()
