#! /usr/bin/python3
#! -*- coding:utf-8; mode:python; -*-

import argparse
import json

def main():
    ap = argparse.ArgumentParser(description="Formats a JSON confusion matrix as TeX code.")
    ap.add_argument('src', metavar='FILE', help="read from FILE")
    ap.add_argument('-o', '--output', dest='dst', metavar='FILE', type=argparse.FileType('w'), help="write to FILE")
    ns = ap.parse_args()
    with open(ns.src, 'r') as istr:
        info = json.load(istr)
    #
    print(r'\def\XValTestRuns{'      + format(info['test-runs' ], ',d').replace(',', '\\,') + '}', file=ns.dst)
    #
    print(r'\def\XValCountApprox{'   + format(round(info['test-count']['mean']), ',d').replace(',', '\\,') + '}',
          file=ns.dst)
    print(r'\def\XValCountMean{'     + format(info['test-count']['mean'  ],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValCountStdev{'    + format(info['test-count']['stdev' ],  '.2f') + '}', file=ns.dst)
    #
    print(r'\def\XValSuccessMean{'   + format(100.0 * info['success' ]['mean' ],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValSuccessStdev{'  + format(100.0 * info['success' ]['stdev'],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValFailureMean{'   + format(100.0 * info['failure' ]['mean' ],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValFailureStdev{'  + format(100.0 * info['failure' ]['stdev'],  '.2f') + '}', file=ns.dst)
    #
    print(r'\def\XValCondPosMean{'   + format(100.0 * info['condition-positive']['mean' ],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValCondPosStdev{'  + format(100.0 * info['condition-positive']['stdev'],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValCondNegMean{'   + format(100.0 * info['condition-negative']['mean' ],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValCondNegStdev{'  + format(100.0 * info['condition-negative']['stdev'],  '.2f') + '}', file=ns.dst)
    #
    print(r'\def\XValPredPosMean{'   + format(100.0 * info['prediction-positive']['mean' ],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValPredPosStdev{'  + format(100.0 * info['prediction-positive']['stdev'],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValPredNegMean{'   + format(100.0 * info['prediction-negative']['mean' ],  '.2f') + '}', file=ns.dst)
    print(r'\def\XValPredNegStdev{'  + format(100.0 * info['prediction-negative']['stdev'],  '.2f') + '}', file=ns.dst)
    #
    print(r'\def\XValTrueNegMean{'   + format(100.0 * info['true-negative' ]['mean' ], '.2f') + '}', file=ns.dst)
    print(r'\def\XValTrueNegStdev{'  + format(100.0 * info['true-negative' ]['stdev'], '.2f') + '}', file=ns.dst)
    print(r'\def\XValFalseNegMean{'  + format(100.0 * info['false-negative']['mean' ], '.2f') + '}', file=ns.dst)
    print(r'\def\XValFalseNegStdev{' + format(100.0 * info['false-negative']['stdev'], '.2f') + '}', file=ns.dst)
    print(r'\def\XValTruePosMean{'   + format(100.0 * info['true-positive' ]['mean' ], '.2f') + '}', file=ns.dst)
    print(r'\def\XValTruePosStdev{'  + format(100.0 * info['true-positive' ]['stdev'], '.2f') + '}', file=ns.dst)
    print(r'\def\XValFalsePosMean{'  + format(100.0 * info['false-positive']['mean' ], '.2f') + '}', file=ns.dst)
    print(r'\def\XValFalsePosStdev{' + format(100.0 * info['false-positive']['stdev'], '.2f') + '}', file=ns.dst)

if __name__ == '__main__':
    main()
