#! /usr/bin/python3
#! -*- coding:utf-8; mode:python; -*-

import argparse
import collections
import json
import os

def main():
    ap = argparse.ArgumentParser(description="Formats JSON confusion matrices with without as TeX code.")
    ap.add_argument('-d', '--directory', metavar='DIR', required=True, help="find and use JSON files in DIR")
    ap.add_argument('-o', '--output', metavar='FILE', type=argparse.FileType('w'), help="write to FILE")
    ns = ap.parse_args()
    data = collections.defaultdict(dict)
    with os.scandir(ns.directory) as dirent:
        for item in dirent:
            if item.is_file():
                for stem in [ 'with', 'without' ]:
                    prefix = 'cross-valid-' + stem + '-'
                    suffix = '.json'
                    if item.name.startswith(prefix) and item.name.endswith(suffix):
                        key = item.name[len(prefix) : -len(suffix)].replace('-', '_').upper()
                        with open(item.path, 'r') as istr:
                            data[key][stem] = info_2_mean_stdev(json.load(istr))
    print_output(data, ns.output)

def info_2_mean_stdev(info):
    return (info['success']['mean'], info['success']['stdev'])

def print_output(data, ostr=None):
    ascending = sorted(data.items(), key=(lambda kv : kv[1]['without'][0]))
    for (key, val) in ascending:
        k = key.replace('_', '\\_')
        mx = format(100.0 * val['without'][0], '.2f') if 'without' in val else ''
        sx = format(100.0 * val['without'][1], '.2f') if 'without' in val else ''
        my = format(100.0 * val['with'   ][0], '.2f') if 'with'    in val else ''
        sy = format(100.0 * val['with'   ][1], '.2f') if 'with'    in val else ''
        print(r"\WithoutResult[\enum{" + k + '}]{' + mx + '}{' + sx + '}{' + my + '}{' + sy + '}', file=ostr)

if __name__ == '__main__':
    main()
