/* -*- coding:utf-8; mode:c++; -*- */

/**
 * @file angular.hxx
 *
 * @brief
 *     Angular all the things.
 *
 */

#ifndef MSC_ANGULAR_HXX
#define MSC_ANGULAR_HXX

#include <vector>

#include "enums/treatments.hxx"
#include "ogdf_fwd.hxx"

namespace msc
{

    /**
     * @brief
     *     Computes all angles between adjacent incident edges.
     *
     * For vertices with degree 1 an angle of 2 &pi; will be reported.
     *
     * @param attrs
     *     layout to operate on
     *
     * @param treatment
     *     strategy for treating non-finite angles
     *
     * @returns
     *     angles in unspecified order
     *
     */
    std::vector<double>
    get_all_angles_between_adjacent_incident_edges(const ogdf::GraphAttributes& attrs,
                                                   treatments treatment = treatments::exception);

}  // namespace msc

#endif  // !defined(MSC_ANGULAR_HXX)
