/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "file.hxx"

#include "iosupp.hxx"
#include "strings.hxx"

namespace msc
{

    file::file() noexcept
    {
    }

    file::file(const std::string_view spec)
    {
        if (!spec.empty()) {
            const auto [destspec, algospec] = split_filename(spec);
            if (is_nullio(destspec)) {
                _terminal = terminals::null;
            } else if (is_stdio(destspec)) {
                _terminal = terminals::stdio;
            } else if (const auto fd = is_fdno(destspec)) {
                _terminal = terminals::descriptor;
                _descriptor = check_descriptor(fd.value());
            } else {
                _terminal = terminals::file;
                _filename = concat(check_filename(destspec));
            }
            const auto compression = algospec.empty() ? compressions::automatic : value_of_compressions(algospec);
            _compression = (compression == compressions::automatic) ? guess_compression(_filename) : compression;
        }
    }

    file file::from_filename(const std::string_view filename, const compressions compression)
    {
        check_filename(filename);
        auto self = file{};
        self._terminal = terminals::file;
        self._compression = (compression == compressions::automatic) ? guess_compression(filename) : compression;
        self._filename.assign(filename.data(), filename.size());
        return self;
    }

    file file::from_descriptor(const int descriptor, const compressions compression)
    {
        check_descriptor(descriptor);
        auto self = file{};
        self._terminal = terminals::descriptor;
        self._compression = (compression == compressions::automatic) ? compressions::none : compression;
        self._descriptor = descriptor;
        return self;
    }

    file file::from_null(const compressions compression)
    {
        auto self = file{};
        self._terminal = terminals::null;
        self._compression = (compression == compressions::automatic) ? compressions::none : compression;
        return self;
    }

    file file::from_stdio(const compressions compression)
    {
        auto self = file{};
        self._terminal = terminals::stdio;
        self._compression = (compression == compressions::automatic) ? compressions::none : compression;
        return self;
    }

    void file::assign_from_spec(const std::string_view spec)
    {
        *this = file{spec};
    }

    int file::mode() const noexcept
    {
        return 0;
    }

    int input_file::mode() const noexcept
    {
        return 'I';
    }

    int output_file::mode() const noexcept
    {
        return 'O';
    }

}  // namespace msc
