/* -*- coding:utf-8; mode:c++; -*- */

#ifndef MSC_INCLUDED_FROM_FILE_HXX
#  error "Never `#include <file.txx>` directly, `#include <file.hxx>` instead"
#endif

#include <utility>

namespace msc
{

    inline terminals file::terminal() const noexcept
    {
        return _terminal;
    }

    inline compressions file::compression() const noexcept
    {
        return _compression;
    }

    inline const std::string& file::filename() const noexcept
    {
        return _filename;
    }

    inline int file::descriptor() const noexcept
    {
        return _descriptor;
    }

    template <terminals Term, typename... ArgTs>
    void file::assign(ArgTs&&... args)
    {
        if constexpr (Term == terminals{}) {
            *this = file{std::forward<ArgTs>(args)...};
        } else if constexpr (Term == terminals::null) {
            *this = file::from_null(std::forward<ArgTs>(args)...);
        } else if constexpr (Term == terminals::stdio) {
            *this = file::from_stdio(std::forward<ArgTs>(args)...);
        } else if constexpr (Term == terminals::file) {
            *this = file::from_filename(std::forward<ArgTs>(args)...);
        } else if constexpr (Term == terminals::descriptor) {
            *this = file::from_descriptor(std::forward<ArgTs>(args)...);
        } else {
            static_assert(Term == terminals{}, "A valid terminal type is required");
        }
    }

    inline bool operator==(const file& lhs, const file& rhs) noexcept
    {
        return (lhs.terminal() == rhs.terminal())
            && (lhs.compression() == rhs.compression())
            && (lhs.filename() == rhs.filename())
            && (lhs.descriptor() == rhs.descriptor());
    }

    inline bool operator!=(const file& lhs, const file& rhs) noexcept
    {
        return !(lhs == rhs);
    }

    inline input_file::input_file(file src) : file{std::move(src)}
    {
    }

    inline output_file::output_file(file dst) : file{std::move(dst)}
    {
    }

}  // namespace msc
