/* -*- coding:utf-8; mode:c++; -*- */

#ifndef MSC_FINGERPRINT_HXX
#define MSC_FINGERPRINT_HXX

#include <string>

#include "ogdf_fwd.hxx"

namespace msc
{

    /**
     * @brief
     *     Returns a fixed-length string that only depends on the given graph and is unlikely to collide with the ID
     *     returned for any other graph.
     *
     * Note that isomorphic graphs with different labels will (almost certainly) not get the same ID.
     *
     * @param graph
     *     graph to obtain an ID for
     *
     * @returns
     *     almost unique ID for the graph
     *
     */
    std::string graph_fingerprint(const ogdf::Graph& graph);

    /**
     * @brief
     *     Returns a fixed-length string that only depends on the given layout and is unlikely to collide with the ID
     *     returned for any other layout.
     *
     * Note that isomorphic graphs with different labels will (almost certainly) not get the same ID.
     *
     * @param attrs
     *     layout to obtain an ID for
     *
     * @returns
     *     almost unique ID for the graph
     *
     */
    std::string layout_fingerprint(const ogdf::GraphAttributes& attrs);

    std::string graph_and_layout_fingerprint(const ogdf::GraphAttributes& attrs);

}  // namespace msc

#endif  // !defined(MSC_FINGERPRINT_HXX)
