/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gliding.hxx"

#include <cassert>
#include <cmath>
#include <utility>

#include <boost/iterator/transform_iterator.hpp>

#include "numeric.hxx"

namespace msc
{

    double get_differential_entropy_of_pdf(const std::vector<std::pair<double, double>>& density)
    {
        assert(std::abs(1.0 - integrate_trapezoidal(std::begin(density), std::end(density))) < 1.0E-3);
        struct boost_wants_no_lambda final
        {
            std::pair<double, double> operator()(const std::pair<double, double> xp) const noexcept
            {
                const auto [x, p] = xp;
                const auto plog2p = (p > 0.0) ? -(p * std::log2(p)) : 0.0;
                return std::make_pair(x, plog2p);
            }
        };
        const auto first = boost::make_transform_iterator(std::begin(density), boost_wants_no_lambda{});
        const auto last = boost::make_transform_iterator(std::end(density), boost_wants_no_lambda{});
        return integrate_trapezoidal(first, last);
    }

}  // namespace msc
