/* -*- coding:utf-8; mode:c++; -*- */

/**
 * @file meta.hxx
 *
 * @brief
 *     Functionality for writing meta data.
 *
 * All tools should support the `--meta` or `-m` option in their CLIs in order to emit meta data in JSON format.  This
 * component provides utilities to handle this in a coherent way.
 *
 */

#ifndef MSC_META_HXX
#define MSC_META_HXX

namespace msc
{

    struct json_object;
    struct output_file;

    /**
     * @brief
     *     Serializes and writes the meta data in `info` to `dest`.
     *
     * @param info
     *     JSON object to serialize and write
     *
     * @param dest
     *     destination to write to
     *
     * @throws std::system_error
     *     if there was an error writing to the file
     *
     */
    void print_meta(const json_object& info, const output_file& dest);

}  // namespace msc

#endif  // !defined(MSC_META_HXX)
