/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "ogdf_fix.hxx"

#include <algorithm>

#include <ogdf/basic/Graph.h>
#include <ogdf/basic/GraphAttributes.h>

namespace msc
{

    msc::point2d get_coords(const ogdf::GraphAttributes& attrs, const ogdf::node v)
    {
        return {attrs.x(v), attrs.y(v)};
    }

    std::pair<msc::point2d, msc::point2d> get_bounding_box(const ogdf::GraphAttributes& attrs) noexcept
    {
        auto sw = make_invalid_point<double, 2>();
        auto ne = make_invalid_point<double, 2>();
        if (attrs.constGraph().numberOfNodes() > 0) {
            sw = get_coords(attrs, attrs.constGraph().firstNode());
            ne = get_coords(attrs, attrs.constGraph().lastNode());
        }
        for (const auto v : attrs.constGraph().nodes) {
            const auto x = attrs.x(v);
            const auto y = attrs.y(v);
            sw.x() = std::min(sw.x(), x);
            sw.y() = std::min(sw.y(), y);
            ne.x() = std::max(ne.x(), x);
            ne.y() = std::max(ne.y(), y);
        }
        return {sw, ne};
    }

    msc::point2d get_bounding_box_size(const ogdf::GraphAttributes& attrs) noexcept
    {
        const auto [sw, ne] = get_bounding_box(attrs);
        return ne - sw;
    }

}  // namespace msc
