/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "pairwise.hxx"

#include <limits>

#include <ogdf/basic/EdgeArray.h>
#include <ogdf/graphalg/ShortestPathAlgorithms.h>

#include "logging.hxx"

// Make sure we've actually included the OGDF headers as advertised in the DocString instead of just forward-declaring
// the types as we usually do.

static_assert(sizeof(ogdf::Graph) > 0);
static_assert(sizeof(ogdf::GraphAttributes) > 0);

namespace msc
{

    namespace /*anonymous*/
    {

        std::unique_ptr<ogdf_node_array_2d<double>>
        make_ogdf_node_array_2d_double(const ogdf::Graph& graph)
        {
            const auto initval = std::numeric_limits<double>::quiet_NaN();
            return std::make_unique<ogdf_node_array_2d<double>>(graph, ogdf_node_array_1d<double>{graph, initval});
        }

    }  // namespace /*anonymous*/

    std::unique_ptr<ogdf_node_array_2d<double>> get_pairwise_shortest_paths(const ogdf::Graph& graph)
    {
        MSC_LOG_INFO("Computing pairwise shortest paths in graph with %d nodes ...", graph.numberOfNodes());
        auto matrix = make_ogdf_node_array_2d_double(graph);
        const auto unitweights = ogdf::EdgeArray<double>{graph, 1.0};
        ogdf::dijkstra_SPAP(graph, *matrix, unitweights);
        return matrix;
    }

}  // namespace msc
