/* -*- coding:utf-8; mode:c++; -*- */

/**
 * @file princomp.hxx
 *
 * @brief
 *     TODO: Explain this header file
 *
 */

#ifndef MSC_PRINCOMP_HXX
#define MSC_PRINCOMP_HXX

#include <array>
#include <cstddef>
#include <iterator>
#include <type_traits>
#include <utility>

#include "point.hxx"

namespace msc
{

    template
    <
        typename EngineT,
        typename FwdIterT,
        typename PointT = typename std::iterator_traits<FwdIterT>::value_type,
        typename T = typename PointT::value_type,
        std::size_t N = std::tuple_size<PointT>::value,
        std::size_t M = N
    >
    std::array<PointT, M> find_primary_axes(const FwdIterT first, const FwdIterT last, EngineT& engine,
                                            std::integral_constant<std::size_t, M> dimensions = {});

    template
    <
        typename EngineT,
        typename ContainerT,
        typename PointT = typename ContainerT::value_type,
        typename T = typename PointT::value_type,
        std::size_t N = std::tuple_size<PointT>::value,
        std::size_t M = N
    >
    std::array<PointT, M> find_primary_axes(ContainerT& coords, EngineT& engine,
                                            std::integral_constant<std::size_t, M> dimensions = {})
    {
        return find_primary_axes(std::begin(coords), std::end(coords), engine, dimensions);
    }

    template
    <
        typename EngineT,
        typename ContainerT,
        typename PointT = typename ContainerT::value_type,
        typename T = typename PointT::value_type,
        std::size_t N = std::tuple_size<PointT>::value,
        std::size_t M = N
    >
    std::array<PointT, M> find_primary_axes_nondestructive(ContainerT coords, EngineT& engine,
                                                           std::integral_constant<std::size_t, M> dimensions = {})
    {
        return find_primary_axes(std::begin(coords), std::end(coords), engine, dimensions);
    }

}  // namespace msc

#define MSC_INCLUDED_FROM_PRINCOMP_HXX
#include "princomp.txx"
#undef MSC_INCLUDED_FROM_PRINCOMP_HXX

#endif  // !defined(MSC_PRINCOMP_HXX)
