/* -*- coding:utf-8; mode:c++; -*- */

#ifndef MSC_INCLUDED_FROM_RANDOM_HXX
#  error "Never `#include <random.txx>` directly; `#include <random.hxx>` instead"
#endif

#include <random>

namespace msc
{

    namespace random_detail
    {

        std::string get_seed_string();

    }  // namespace random_detail

    template <typename EngT>
    std::string seed_random_engine(EngT& engine)
    {
        const auto seed = random_detail::get_seed_string();
        auto seedseq = std::seed_seq(seed.begin(), seed.end());
        engine.seed(seedseq);
        return seed;
    }

    template <typename EngineT>
    std::string random_hex_string(EngineT& engine, const std::size_t bytes)
    {
        // NB: We do not use a distribution here because unlike for engines, the ISO C++ standard does not specify the
        // sequence of numbers they produce.
        static const char hexdigits[] = "0123456789abcdef";
        auto result = std::string{};
        result.reserve(2 * bytes);
        for (auto i = std::size_t{}; i < bytes; ++i) {
            const auto byte = engine() & 0xffU;
            const auto lo = ((byte >> 0) & 0xfU);
            const auto hi = ((byte >> 4) & 0xfU);
            result.push_back(hexdigits[hi]);
            result.push_back(hexdigits[lo]);
        }
        return result;
    }

}  // namespace msc
