/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <cstddef>
#include <utility>
#include <vector>

#include <ogdf/basic/Graph.h>
#include <ogdf/basic/GraphAttributes.h>

#include "angular.hxx"
#include "cli.hxx"
#include "data_analysis.hxx"
#include "io.hxx"
#include "json.hxx"
#include "meta.hxx"
#include "useful.hxx"

#define PROGRAM_NAME "angular"

namespace /*anonymous*/
{

    struct application final
    {
        msc::cli_parameters_property parameters{};
        void operator()() const;
    };

    msc::json_object basic_info()
    {
        auto info = msc::json_object{};
        info["producer"] = PROGRAM_NAME;
        return info;
    }

    void application::operator()() const
    {
        const auto [graph, attrs] = msc::load_layout(this->parameters.input);
        auto info = basic_info();
        auto subinfos = msc::json_array{};
        auto angles = msc::get_all_angles_between_adjacent_incident_edges(*attrs, msc::treatments::ignore);
        auto analyzer = msc::data_analyzer{this->parameters.kernel};
        auto entropies = msc::initialize_entropies();
        analyzer.set_range(0.0, 2.0 * M_PI);
        for (std::size_t i = 0; i < this->parameters.iterations(); ++i) {
            auto subinfo = msc::json_object{};
            analyzer.set_width(msc::get_item(this->parameters.width, i));
            analyzer.set_bins(msc::get_item(this->parameters.bins, i));
            analyzer.set_points(this->parameters.points);
            analyzer.set_output(this->parameters.expand_filename(i));
            analyzer.analyze(std::begin(angles), std::end(angles), info, subinfo);
            msc::append_entropy(entropies, subinfo, "bincount");
            subinfos.push_back(std::move(subinfo));
        }
        info["data"] = std::move(subinfos);
        msc::assign_entropy_regression(entropies, info);
        msc::print_meta(info, this->parameters.meta);
        (void) graph.get();
    }

}  // namespace /*anonymous*/

int main(const int argc, const char *const *const argv)
{
    auto app = msc::command_line_interface<application>{PROGRAM_NAME};
    app.help.push_back("Computes the distribution of angles between incident edges.");
    app.help.push_back(msc::helptext_file_name_expansion());
    return app(argc, argv);
}
