/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <chrono>
#include <cstdlib>
#include <exception>
#include <iostream>
#include <thread>

#include "benchmark.hxx"

#define PROGRAM_NAME "sleepy"

namespace /*anonymous*/
{

    void benchmark()
    {
        const auto duration = std::chrono::microseconds{1};
        std::this_thread::sleep_for(duration);
    }

}  // namespace /*anonymous*/

int main(const int argc, const char *const *const argv)
{
    try {
        const auto t0 = msc::benchmark::clock_type::now();
        auto setup = msc::benchmark::benchmark_setup(
            PROGRAM_NAME,
            "Pseudo micro benchmark that artificially delays for 1 microsecond"
        );
        if (!setup.process(argc, argv)) {
            return EXIT_SUCCESS;
        }
        auto constr = setup.get_constraints();
        if (constr.timeout.count() > 0) {
            constr.timeout -= msc::benchmark::duration_type{msc::benchmark::clock_type::now() - t0};
        }
        const auto result = msc::benchmark::run_benchmark(constr, benchmark);
        msc::benchmark::print_result(result);
        return EXIT_SUCCESS;
    } catch (const std::exception& e) {
        std::cerr << PROGRAM_NAME << ": error: " << e.what() << std::endl;
        return EXIT_FAILURE;
    }
}
