/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define MSC_RUN_ALL_UNIT_TESTS_IN_MAIN

#include "gliding.hxx"

#include <utility>
#include <vector>

#include "stochastic.hxx"
#include "unittest.hxx"

namespace /*anonymous*/
{

    // https://de.wikipedia.org/wiki/Differentielle_Entropie#Differentielle_Entropie_f%C3%BCr_verschiedene_Verteilungen

    MSC_AUTO_TEST_CASE(get_differential_entropy_of_pdf_uniform_distribution)
    {
        for (auto width = 0.5; width < 3.0; width += 0.5) {
            const auto mean = -13.5;
            auto samples = std::vector<std::pair<double, double>>{};
            for (auto x = mean - width; x < mean + width; x += 0.01 * width) {
                const auto y = ((x >= mean - width / 2.0) && (x <= mean + width / 2.0)) ? 1.0 / width : 0.0;
                samples.emplace_back(x, y);
            }
            MSC_REQUIRE_LT(samples.front().second, 1.0E-10);
            MSC_REQUIRE_LT(samples.back().second, 1.0E-10);
            const auto expected = 0.5 * std::log2(width * width);
            const auto actual = msc::get_differential_entropy_of_pdf(samples);
            MSC_REQUIRE_CLOSE(1.0E-10, expected, actual);
        }
    }

    MSC_AUTO_TEST_CASE(get_differential_entropy_of_pdf_normal_distribution)
    {
        for (auto sigma = 0.125; sigma < 8.0; sigma *= 2.0) {
            const auto mu = 42.0;
            const auto gaussian = msc::gaussian{mu, sigma};
            auto samples = std::vector<std::pair<double, double>>{};
            for (auto x = mu - 10.0 * sigma; x <= mu + 10.0 * sigma; x += 0.1 * sigma) {
                samples.emplace_back(x, gaussian(x));
            }
            MSC_REQUIRE_LT(samples.front().second, 1.0E-10);
            MSC_REQUIRE_LT(samples.back().second, 1.0E-10);
            const auto expected = 0.5 * std::log2(2.0 * M_PI * M_E * sigma * sigma);
            const auto actual = msc::get_differential_entropy_of_pdf(samples);
            MSC_REQUIRE_CLOSE(1.0E-10, 1.0, actual / expected);
        }
    }

}  // namespace /*anonymous*/
