/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define MSC_RUN_ALL_UNIT_TESTS_IN_MAIN

#include "kernel.hxx"

#include <algorithm>
#include <set>
#include <utility>

#include "unittest.hxx"

namespace /*anonymous*/
{

    MSC_AUTO_TEST_CASE(all_kernels)
    {
        const auto themall = msc::all_kernels();
        MSC_REQUIRE(std::is_sorted(std::begin(themall), std::end(themall)));
        MSC_REQUIRE(std::find(std::begin(themall), std::end(themall), msc::kernels{}) == std::end(themall));
    }

    MSC_AUTO_TEST_CASE(kernels_name)
    {
        MSC_REQUIRE_EQ("boxed", name(msc::kernels::boxed));
        MSC_REQUIRE_EQ("gaussian", name(msc::kernels::gaussian));
        MSC_REQUIRE_EQ("raw", name(msc::kernels::raw));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, name(msc::kernels{}));
    }

    MSC_AUTO_TEST_CASE(kernels_name_unique)
    {
        auto names = std::set<std::string_view>{};
        for (const auto kernel : msc::all_kernels()) {
            names.insert(name(kernel));
        }
        MSC_REQUIRE_EQ(msc::all_kernels().size(), names.size());
    }

    MSC_AUTO_TEST_CASE(value_of_kernel)
    {
        MSC_REQUIRE_EQ(msc::kernels::boxed, msc::value_of_kernel("boxed"));
        MSC_REQUIRE_EQ(msc::kernels::gaussian, msc::value_of_kernel("gaussian"));
        MSC_REQUIRE_EQ(msc::kernels::raw, msc::value_of_kernel("raw"));
        MSC_REQUIRE_EQ(msc::kernels::boxed, msc::value_of_kernel("Boxed"));
        MSC_REQUIRE_EQ(msc::kernels::boxed, msc::value_of_kernel("BoxeD"));
        MSC_REQUIRE_EQ(msc::kernels::boxed, msc::value_of_kernel("BOXED"));
        MSC_REQUIRE_EQ(msc::kernels::boxed, msc::value_of_kernel("  BOXED  \t\t "));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, msc::value_of_kernel(""));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, msc::value_of_kernel("\t \t"));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, msc::value_of_kernel("Hazlenut"));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, msc::value_of_kernel(""));
    }

    MSC_AUTO_TEST_CASE(value_of_kernel_textual_round_trip)
    {
        for (const auto kernel : msc::all_kernels()) {
            MSC_REQUIRE_EQ(kernel, msc::value_of_kernel(name(kernel)));
        }
    }

    MSC_AUTO_TEST_CASE(value_of_kernel_numeric_round_trip)
    {
        for (const auto kernel : msc::all_kernels()) {
            const auto raw = static_cast<int>(kernel);
            MSC_REQUIRE_EQ(kernel, msc::value_of_kernel(std::to_string(raw)));
        }
    }

}  // namespace /*anonymous*/
