/* -*- coding:utf-8; mode:c++; -*- */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define MSC_RUN_ALL_UNIT_TESTS_IN_MAIN

#include "layouts.hxx"

#include <algorithm>
#include <set>
#include <utility>

#include "unittest.hxx"

namespace /*anonymous*/
{

    MSC_AUTO_TEST_CASE(all_algorithms)
    {
        const auto themall = msc::all_algorithms();
        MSC_REQUIRE(std::is_sorted(std::begin(themall), std::end(themall)));
        MSC_REQUIRE(std::find(std::begin(themall), std::end(themall), msc::algorithms{}) == std::end(themall));
    }

    MSC_AUTO_TEST_CASE(algorithms_name)
    {
        MSC_REQUIRE_EQ("fmmm", name(msc::algorithms::fmmm));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, name(msc::algorithms{}));
    }

    MSC_AUTO_TEST_CASE(algorithms_name_unique)
    {
        auto names = std::set<std::string_view>{};
        for (const auto algo : msc::all_algorithms()) {
            names.insert(name(algo));
        }
        MSC_REQUIRE_EQ(msc::all_algorithms().size(), names.size());
    }

    MSC_AUTO_TEST_CASE(value_of_algorithm)
    {
        MSC_REQUIRE_EQ(msc::algorithms::fmmm, msc::value_of_algorithm("fmmm"));
        MSC_REQUIRE_EQ(msc::algorithms::fmmm, msc::value_of_algorithm("FMMM"));
        MSC_REQUIRE_EQ(msc::algorithms::fmmm, msc::value_of_algorithm("  Fmmm  \t"));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, msc::value_of_algorithm(""));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, msc::value_of_algorithm("\t \t"));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, msc::value_of_algorithm("Hazlenut"));
        MSC_REQUIRE_EXCEPTION(std::invalid_argument, msc::value_of_algorithm(""));
    }

    MSC_AUTO_TEST_CASE(value_of_algorithm_textual_round_trip)
    {
        for (const auto algo : msc::all_algorithms()) {
            MSC_REQUIRE_EQ(algo, msc::value_of_algorithm(name(algo)));
        }
    }

    MSC_AUTO_TEST_CASE(value_of_algorithm_numeric_round_trip)
    {
        for (const auto algo : msc::all_algorithms()) {
            const auto raw = static_cast<int>(algo);
            MSC_REQUIRE_EQ(algo, msc::value_of_algorithm(std::to_string(raw)));
        }
    }

}  // namespace /*anonymous*/
